#!/bin/bash
################################################################################
# Intended use: Compile and extract software packages
#
# Notes: This script support Linux Bash shell script only
# Install Bash from https://www.gnu.org/software/bash/
#
# Copyright (C) 2015 - 2023, VGISC Dev Team <dev@vgisc.com>
################################################################################

# Source path
export SCRIPT=$(readlink -f "$0")
export SCRIPT_PATH=$(dirname "$SCRIPT")
export SRC_DIR=${SCRIPT_PATH}/../..
export APP_NAME=$(basename $(dirname $(dirname "${SCRIPT_PATH}")))

# Declare input
DEVICE_TYPE=linux-x86_64
BRANCH=mmds
VERSION="3.3.2"

export BUILD_TYPE=static

# *********************************************************************
# Function: print_help
# 	- Print: Help
# *********************************************************************
print_help ()
{	
	# Print
	echo -e ""
	echo -e "Syntax: build.sh <arch> <version>"
	echo -e "<arch>: Arch of device hardware. Only support"
	echo -e "	linux-arm32 	Linux ARM 32 bit"
	echo -e "	linux-arm64 	Linux ARM 64 bit"
	echo -e "	linux-ppc64 	Linux PowerPC 64 bit"
	echo -e "	linux-x86_64 	Linux AMD 64 bit"
	echo -e "	windows-mingw32	Windows AMD 32 bit"
	echo -e "	windows-mingw64	Windows AMD 64 bit"
	echo -e "	linux-kirkwood	OpenWRT Kirkwood"
	echo -e "<branch>: Source branch of software. Only support"
	echo -e "	bml 		Branch for BML"
	echo -e "	mmds 		Branch for MMDS"
	echo -e "	tmc 		Branch for TMC"
	echo -e "	net 		Branch for NET"
	echo -e ""
	echo -e "Example:"
	echo -e "- Example: build.sh linux-arm32 bml"
	echo -e "- Example: build.sh \"linux-ppc64\" \"mmds\""
	echo -e ""
}

# Validate input
if [[ -z $DEVICE_TYPE || -z $BRANCH ]]; then
	echo -e "You must enter enough information. Cannot execute the command."
	print_help
	exit
fi

# Variables
item_exists="0"
names=("linux-arm32" \
"linux-arm64" \
"linux-ppc64" \
"linux-x86_64" \
"windows-mingw32" \
"windows-mingw64" \
"linux-kirkwood")
for item in ${names[@]}; do
	if [[ "${item}" = "${DEVICE_TYPE}" ]]; then
    	item_exists="1"
	fi
done
if [[ "$item_exists" == "0" ]]; then
	echo -e "The device type is invalid. Cannot execute the command."
	echo -e "Device type must be in supported list."
	print_help
	exit
fi

item_exists="0"
names=("bml" \
"mmds" \
"tmc" \
"net")
for item in ${names[@]}; do
	if [[ "${item}" = "${BRANCH}" ]]; then
    	item_exists="1"
	fi
done
if [[ "$item_exists" == "0" ]]; then
	echo -e "The software branch is invalid. Cannot execute the command."
	echo -e "Software branch must be in supported list."
	print_help
	exit
fi

echo ""
echo "==========================================================================="
echo "Build ${APP_NAME} for device platform ${DEVICE_TYPE} - branch ${BRANCH} - version $VERSION"
echo "Build type: ${BUILD_TYPE}"
echo "=========================================================================="

echo "=> Setting up the build environment for ${DEVICE_TYPE}..."
unset LD_LIBRARY_PATH
case "${DEVICE_TYPE}" in
linux-arm32)
	export PATH="/opt/Xilinx/arm-buildroot-linux-gnueabihf_sdk/bin:$PATH"
	export SYS_ROOT=/opt/Xilinx/arm-buildroot-linux-gnueabihf_sdk/usr/arm-buildroot-linux-gnueabihf/sysroot/usr
	export OS=linux
	export ARCH=arm32
	export TARGET=arm-linux-gnueabihf
	export HOST=arm-linux-gnueabihf
	export CC=arm-linux-gnueabihf-gcc
	export LD=arm-linux-gnueabihf-ld
	export RANLIB=arm-linux-gnueabihf-ranlib
	export AR=arm-linux-gnueabihf-ar
	export STRIP=arm-linux-gnueabihf-strip

	gcc_bin="$(which arm-linux-gnueabihf-gcc)"
	if [[ ! -f "${gcc_bin}" ]]; then
		echo -e "GCC compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
	gpp_bin=$(which arm-linux-gnueabihf-g++)
	if [[ ! -f "${gpp_bin}" ]]; then
		echo -e "G++ compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
    ;;
linux-arm64)
	export PATH="/opt/Xilinx/aarch64-buildroot-linux-gnu_sdk/bin:$PATH"
	export SYS_ROOT=/opt/Xilinx/aarch64-buildroot-linux-gnu_sdk/usr/aarch64-buildroot-linux-gnu/sysroot/usr
	export OS=linux
	export ARCH=arm64
	export TARGET=aarch64-linux-gnu
	export HOST=aarch64-linux-gnu
	export CC=aarch64-linux-gnu-gcc
	export LD=aarch64-linux-gnu-ld
	export RANLIB=aarch64-linux-gnu-ranlib
	export AR=aarch64-linux-gnu-ar
	export STRIP=aarch64-linux-gnu-strip

	gcc_bin="$(which aarch64-linux-gnu-gcc)"
	if [[ ! -f "${gcc_bin}" ]]; then
		echo -e "GCC compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
	gpp_bin=$(which aarch64-linux-gnu-g++)
	if [[ ! -f "${gpp_bin}" ]]; then
		echo -e "G++ compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
    ;;
linux-ppc64)
	. /opt/fsl-qoriq/3.0/environment-setup-ppc64e6500-fsl-linux
	export SYSROOT_ALT=/opt/fsl-qoriq/3.0/sysroots/ppc64e6500-fsl-linux/usr
	export OS=linux
	export ARCH=ppc64
	export TARGET=powerpc64-fsl-linux
	export HOST=powerpc64-fsl-linux
	export STRIP=powerpc64-fsl-linux-strip

	gcc_bin="$(which powerpc64-fsl-linux-gcc)"
	if [[ ! -f "${gcc_bin}" ]]; then
		echo -e "GCC compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
	gpp_bin=$(which powerpc64-fsl-linux-g++)
	if [[ ! -f "${gpp_bin}" ]]; then
		echo -e "G++ compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
    ;;
linux-x86_64)
	export OS=linux
	export ARCH=x86_64
	export TARGET=linux-x86_64
	export HOST=linux-x86_64
	export STRIP=strip

	gcc_bin="$(which gcc)"
	if [[ ! -f "${gcc_bin}" ]]; then
		echo -e "GCC compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
	gpp_bin=$(which g++)
	if [[ ! -f "${gpp_bin}" ]]; then
		echo -e "G++ compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
    ;;
windows-mingw32)
	export OS=windows
	export ARCH=mingw32
	export TARGET=i686-w64-mingw32
	export HOST=i686-w64-mingw32
	export CC=i686-w64-mingw32-gcc
	export LD=i686-w64-mingw32-ld
	export RANLIB=i686-w64-mingw32-ranlib
	export AR=i686-w64-mingw32-ar
	export STRIP=i686-w64-mingw32-strip

	gcc_bin="$(which i686-w64-mingw32-gcc)"
	if [[ ! -f "${gcc_bin}" ]]; then
		echo -e "GCC compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
	gpp_bin=$(which i686-w64-mingw32-g++)
	if [[ ! -f "${gpp_bin}" ]]; then
		echo -e "G++ compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
    ;;
windows-mingw64)
	export OS=windows
	export ARCH=mingw64
	export TARGET=x86_64-w64-mingw32
	export HOST=x86_64-w64-mingw32
	export CC=x86_64-w64-mingw32-gcc
	export LD=x86_64-w64-mingw32-ld
	export RANLIB=x86_64-w64-mingw32-ranlib
	export AR=x86_64-w64-mingw32-ar
	export STRIP=x86_64-w64-mingw32-strip

	gcc_bin="$(which x86_64-w64-mingw32-gcc)"
	if [[ ! -f "${gcc_bin}" ]]; then
		echo -e "GCC compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
	gpp_bin=$(which x86_64-w64-mingw32-g++)
	if [[ ! -f "${gpp_bin}" ]]; then
		echo -e "G++ compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
    ;;
linux-kirkwood)
	export PATH="/opt/openwrt/arm_xscale_gcc-11.2.0_musl_eabi/bin:$PATH"
	export SYS_ROOT=/opt/openwrt/arm_xscale_gcc-11.2.0_musl_eabi
	export STAGING_DIR=/opt/openwrt/
	export OS=linux
	export ARCH=arm32
	export TARGET=arm-openwrt-linux
	export HOST=arm-openwrt-linux
	export CC=arm-openwrt-linux-gcc
	export LD=arm-openwrt-linux-ld
	export RANLIB=arm-openwrt-linux-ranlib
	export AR=arm-openwrt-linux-ar
	export STRIP=arm-openwrt-linux-strip

	gcc_bin="$(which arm-openwrt-linux-gcc)"
	if [[ ! -f "${gcc_bin}" ]]; then
		echo -e "GCC compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
	gpp_bin=$(which arm-openwrt-linux-g++)
	if [[ ! -f "${gpp_bin}" ]]; then
		echo -e "G++ compiler for ${DEVICE_TYPE} does not exist. Exit."
		exit
	fi
    ;;
*)
	echo "The build environment cannot be determined. Exit"
	exit
	;;
esac

# Output path
export PREFIX=/opt/vgisc/usr
export THIRD_PARTY_PREFIX=/opt/third_party
export OUTPUT_DIR=output/${APP_NAME}-${DEVICE_TYPE}-${VERSION}-${BUILD_TYPE}
export RELEASE_DIR=$SRC_DIR/../../release/${BRANCH}/release/${DEVICE_TYPE}
export ZIP_DIR=$SRC_DIR/../../release/${BRANCH}/zip
export STATIC_DIR=$SRC_DIR/../../release/${BRANCH}/static/${DEVICE_TYPE}
export DYNAMIC_DIR=$SRC_DIR/../../release/${BRANCH}/dynamic/${DEVICE_TYPE}
export BOOT_DIR=$SRC_DIR/../../release/${BRANCH}/rootfs/boot
export INSTALLERS_DIR=$SRC_DIR/../../release/${BRANCH}/installers/${DEVICE}_installer-${VERSION}
export INSTALLERS_DST=${DEVICE}_installer-${BRANCH}

export DST_DIR=${INSTALLERS_DIR}/root/app/

# Dependences sources path
export ZLIB_SRC=${SRC_DIR}/../../zlib
ZLIB_LIB=""

case "${DEVICE_TYPE}" in
windows-mingw32)
	ZLIB_LIB="zlib1.lib"
    ;;
windows-mingw64)
	ZLIB_LIB="zlib1.lib"
    ;;
*)
	ZLIB_LIB="libz.a"
	;;
esac

export ZLIB_LIB=${ZLIB_LIB}

echo "Building dependences..."

echo ""
echo "=> Change to source directory: ${SRC_DIR}/"
cd ${SRC_DIR}

echo ""
echo "=> Clean output and old build"
#make clean
#make distclean
rm -rf ${SRC_DIR}/${OUTPUT_DIR}
rm -rf ${SRC_DIR}/${OUTPUT_DIR}.tar.gz

echo ""
echo "=> Build in source tree: ${SRC_DIR}/"
export CROSS_COMPILE=
case "${DEVICE_TYPE}" in
linux-x86_64)
./Configure linux-x86_64 no-shared \
	--prefix=${PREFIX} \
	--openssldir=${ETC_DIR} \
	no-tests no-asm
#make update && make depend
    ;;
linux-arm32)	
./Configure linux-armv4 no-shared \
	--prefix=${PREFIX} \
	--openssldir=${ETC_DIR} \
	no-tests no-asm 
make update && make depend
    ;;
linux-arm64)	
./Configure linux-aarch64 no-shared \
	--prefix=${PREFIX} \
	--openssldir=${ETC_DIR}  \
	no-tests no-asm
make update && make depend
    ;;
linux-ppc64)	
./Configure linux-ppc64 no-shared \
	--prefix=${PREFIX} \
	--openssldir=${ETC_DIR} \
	no-tests no-asm
make update && make depend
    ;;
windows-mingw32)
export CC=
export LD=
export RANLIB=
export AR=
./Configure mingw no-shared \
no-tests \
-lws2_32 \
no-dso no-capieng no-autoload-config \
--prefix=${PREFIX} \
--cross-compile-prefix=i686-w64-mingw32-

perl util/mkdef.pl crypto ssl NT update
make depend
    ;;
windows-mingw64)
export CC=
export LD=
export RANLIB=
export AR=	
./Configure mingw64 no-shared \
no-tests \
-lws2_32 \
no-dso no-capieng no-autoload-config \
--prefix=${PREFIX} \
--cross-compile-prefix=x86_64-w64-mingw32-

perl util/mkdef.pl crypto ssl NT update
make depend
    ;;
*)
./Configure no-shared \
	--prefix=${PREFIX} \
	--megassldir=${ETC_DIR}  \
	no-tests no-asm
make update && make depend
    ;;
esac

make -j8
make -j8 DESTDIR=${SRC_DIR}/${OUTPUT_DIR} install_sw

echo ""
echo "=> Make output tar.gz file: ${SRC_DIR}/${OUTPUT_DIR}.tar.gz"
rm -rf ${SRC_DIR}/${OUTPUT_DIR}.tar.gz
mkdir -p ${SRC_DIR}/${OUTPUT_DIR}/${PREFIX}/lib/
if [[ -d "${SRC_DIR}/${OUTPUT_DIR}/${PREFIX}/lib64" ]]; then
	cp -rf ${SRC_DIR}/${OUTPUT_DIR}/${PREFIX}/lib64/* ${SRC_DIR}/${OUTPUT_DIR}/${PREFIX}/lib/
fi
if [[ -f "${SRC_DIR}/${OUTPUT_DIR}/${PREFIX}/bin/libcrypto-3.dll" ]]; then
	cp -rf ${SRC_DIR}/${OUTPUT_DIR}/${PREFIX}/bin/*.dll ${SRC_DIR}/${OUTPUT_DIR}/${PREFIX}/lib/
fi
tar -zcvf ${SRC_DIR}/${OUTPUT_DIR}.tar.gz ${SRC_DIR}/${OUTPUT_DIR}/

echo ""
echo "=> Copy output to static directory: ${STATIC_DIR}${PREFIX}/"
mkdir -p ${STATIC_DIR}
cp -rf ${SRC_DIR}/${OUTPUT_DIR}/* ${STATIC_DIR}/

echo ""
echo "=> Copy output ${OUTPUT_DIR}.tar.gz to compress directory: ${ZIP_DIR}/"
mkdir -p ${ZIP_DIR}/
cp -rf ${SRC_DIR}/${OUTPUT_DIR}.tar.gz ${ZIP_DIR}/

echo ""
echo "=> Clean the build"
#make clean
#make distclean
rm -rf ${SRC_DIR}/${OUTPUT_DIR}
rm -rf ${SRC_DIR}/${OUTPUT_DIR}.tar.gz

echo ""
echo "=> Done."
echo ""
################################################################################
#					BASH SCRIPT ON LINUX/UNIX - END
################################################################################